@@ -1,5 +1,6 @@
 #include "monster_test.h"
 
+#include <limits>
 #include <vector>
 
 #include "flatbuffers/flatbuffer_builder.h"
@@ -421,7 +422,7 @@ void MutateFlatBuffersTest(uint8_t *flatbuf, std::size_t length) {
 
   // Mutate structs.
   auto pos = monster->mutable_pos();
-  auto test3 = pos->mutable_test3();  // Struct inside a struct.
+  auto & test3 = pos->mutable_test3();  // Struct inside a struct.
   test3.mutate_a(50);                 // Struct fields never fail.
   TEST_EQ(test3.a(), 50);
   test3.mutate_a(10);
@@ -507,7 +508,7 @@ void ObjectFlatBuffersTest(uint8_t *flatbuf) {
   CheckMonsterObject(monster2.get());
 
   // Test object copy.
-  auto monster3 = *monster2;
+  MonsterT monster3 = *monster2;
   flatbuffers::FlatBufferBuilder fbb3;
   fbb3.Finish(CreateMonster(fbb3, &monster3, &rehasher), MonsterIdentifier());
   const auto len3 = fbb3.GetSize();
@@ -625,7 +626,7 @@ void TestMonsterExtraFloats(const std::string &tests_data_path) {
   TEST_EQ(def_extra->d3(), -infinity_d);
   std::string jsongen;
   auto result = GenerateText(parser, def_obj, &jsongen);
-  TEST_EQ(result, true);
+  TEST_NULL(result);
   // Check expected default values.
   TEST_EQ(std::string::npos != jsongen.find("f0: nan"), true);
   TEST_EQ(std::string::npos != jsongen.find("f1: nan"), true);
@@ -776,7 +777,7 @@ void ParseAndGenerateTextTest(const std::string &tests_data_path, bool binary) {
   std::string jsongen;
   auto result =
       GenerateText(parser, parser.builder_.GetBufferPointer(), &jsongen);
-  TEST_EQ(result, true);
+  TEST_NULL(result);
   TEST_EQ_STR(jsongen.c_str(), jsonfile.c_str());
 
   // We can also do the above using the convenient Registry that knows about
@@ -814,9 +815,8 @@ void ParseAndGenerateTextTest(const std::string &tests_data_path, bool binary) {
   // request natural printing for utf-8 strings
   parser.opts.natural_utf8 = true;
   parser.opts.strict_json = true;
-  TEST_EQ(
-      GenerateText(parser, parser.builder_.GetBufferPointer(), &jsongen_utf8),
-      true);
+  TEST_NULL(
+      GenerateText(parser, parser.builder_.GetBufferPointer(), &jsongen_utf8));
   TEST_EQ_STR(jsongen_utf8.c_str(), jsonfile_utf8.c_str());
 }
 
@@ -852,4 +852,4 @@ void UnPackTo(const uint8_t *flatbuf) {
 }
 
 }  // namespace tests
-}  // namespace flatbuffers
\ No newline at end of file
+}  // namespace flatbuffers
