@@ -25,6 +25,7 @@
 #include "flatbuffers/flatbuffers.h"
 #include "flatbuffers/idl.h"
 #include "flatbuffers/minireflect.h"
+#include "flatbuffers/reflection_generated.h"
 #include "flatbuffers/registry.h"
 #include "flatbuffers/util.h"
 #include "fuzz_test.h"
@@ -120,12 +121,12 @@ void GenerateTableTextTest(const std::string &tests_data_path) {
   std::string jsongen;
   auto result = GenerateTextFromTable(parser, monster, "MyGame.Example.Monster",
                                       &jsongen);
-  TEST_EQ(result, true);
+  TEST_NULL(result);
   // Test sub table
   const Vec3 *pos = monster->pos();
   jsongen.clear();
   result = GenerateTextFromTable(parser, pos, "MyGame.Example.Vec3", &jsongen);
-  TEST_EQ(result, true);
+  TEST_NULL(result);
   TEST_EQ_STR(
       jsongen.c_str(),
       "{x: 1.0,y: 2.0,z: 3.0,test1: 3.0,test2: \"Green\",test3: {a: 5,b: 6}}");
@@ -133,13 +134,13 @@ void GenerateTableTextTest(const std::string &tests_data_path) {
   jsongen.clear();
   result =
       GenerateTextFromTable(parser, &test3, "MyGame.Example.Test", &jsongen);
-  TEST_EQ(result, true);
+  TEST_NULL(result);
   TEST_EQ_STR(jsongen.c_str(), "{a: 5,b: 6}");
   const Test *test4 = monster->test4()->Get(0);
   jsongen.clear();
   result =
       GenerateTextFromTable(parser, test4, "MyGame.Example.Test", &jsongen);
-  TEST_EQ(result, true);
+  TEST_NULL(result);
   TEST_EQ_STR(jsongen.c_str(), "{a: 10,b: 20}");
 }
 
@@ -337,7 +338,7 @@ void UnionVectorTest(const std::string &tests_data_path) {
   // Generate text using parsed schema.
   std::string jsongen;
   auto result = GenerateText(parser, fbb.GetBufferPointer(), &jsongen);
-  TEST_EQ(result, true);
+  TEST_NULL(result);
   TEST_EQ_STR(jsongen.c_str(),
               "{\n"
               "  main_character_type: \"Rapunzel\",\n"
@@ -912,7 +913,7 @@ void NativeTypeTest() {
 // Guard against -Wunused-function on platforms without file tests.
 #ifndef FLATBUFFERS_NO_FILE_TESTS
 // VS10 does not support typed enums, exclude from tests
-#if !defined(_MSC_VER) || _MSC_VER >= 1700
+#  if !defined(_MSC_VER) || _MSC_VER >= 1700
 void FixedLengthArrayJsonTest(const std::string &tests_data_path, bool binary) {
   // load FlatBuffer schema (.fbs) and JSON from disk
   std::string schemafile;
@@ -955,9 +956,8 @@ void FixedLengthArrayJsonTest(const std::string &tests_data_path, bool binary) {
 
   // Export to JSON
   std::string jsonGen;
-  TEST_EQ(
-      GenerateText(parserOrg, parserOrg.builder_.GetBufferPointer(), &jsonGen),
-      true);
+  TEST_NULL(
+      GenerateText(parserOrg, parserOrg.builder_.GetBufferPointer(), &jsonGen));
 
   // Import from JSON
   TEST_EQ(parserGen.Parse(jsonGen.c_str()), true);
@@ -1032,7 +1032,7 @@ void FixedLengthArraySpanTest(const std::string &tests_data_path) {
         std::equal(const_d_c.begin(), const_d_c.end(), mutable_d_c.begin()));
   }
   // test little endian array of int32
-#  if FLATBUFFERS_LITTLEENDIAN
+#    if FLATBUFFERS_LITTLEENDIAN
   {
     flatbuffers::span<const int32_t, 2> const_d_a =
         flatbuffers::make_span(*const_nested.a());
@@ -1047,12 +1047,12 @@ void FixedLengthArraySpanTest(const std::string &tests_data_path) {
     TEST_ASSERT(
         std::equal(const_d_a.begin(), const_d_a.end(), mutable_d_a.begin()));
   }
-#  endif
+#    endif
 }
-#else
+#  else
 void FixedLengthArrayJsonTest(bool /*binary*/) {}
 void FixedLengthArraySpanTest() {}
-#endif
+#  endif
 
 void TestEmbeddedBinarySchema(const std::string &tests_data_path) {
   // load JSON from disk
@@ -1082,9 +1082,8 @@ void TestEmbeddedBinarySchema(const std::string &tests_data_path) {
 
   // Export to JSON
   std::string jsonGen;
-  TEST_EQ(
-      GenerateText(parserOrg, parserOrg.builder_.GetBufferPointer(), &jsonGen),
-      true);
+  TEST_NULL(
+      GenerateText(parserOrg, parserOrg.builder_.GetBufferPointer(), &jsonGen));
 
   // Import from JSON
   TEST_EQ(parserGen.Parse(jsonGen.c_str()), true);
@@ -1103,6 +1102,37 @@ void TestEmbeddedBinarySchema(const std::string &tests_data_path) {
 }
 #endif
 
+template<typename T> void EmbeddedSchemaAccessByType() {
+  // Get the binary schema from the Type itself.
+  // Verify the schema is OK.
+  flatbuffers::Verifier verifierEmbeddedSchema(
+      T::TableType::BinarySchema::data(), T::TableType::BinarySchema::size());
+  TEST_EQ(reflection::VerifySchemaBuffer(verifierEmbeddedSchema), true);
+
+  // Reflect it.
+  auto schema = reflection::GetSchema(T::TableType::BinarySchema::data());
+
+  // This should equal the expected root table.
+  TEST_EQ_STR(schema->root_table()->name()->c_str(), "MyGame.Example.Monster");
+}
+
+void EmbeddedSchemaAccess() {
+  // Get the binary schema for the monster.
+  // Verify the schema is OK.
+  flatbuffers::Verifier verifierEmbeddedSchema(Monster::BinarySchema::data(),
+                                               Monster::BinarySchema::size());
+  TEST_EQ(reflection::VerifySchemaBuffer(verifierEmbeddedSchema), true);
+
+  // Reflect it.
+  auto schema = reflection::GetSchema(Monster::BinarySchema::data());
+
+  // This should equal the expected root table.
+  TEST_EQ_STR(schema->root_table()->name()->c_str(), "MyGame.Example.Monster");
+
+  // Repeat above, but do so through a template parameter:
+  EmbeddedSchemaAccessByType<StatT>();
+}
+
 void NestedVerifierTest() {
   // Create a nested monster.
   flatbuffers::FlatBufferBuilder nested_builder;
@@ -1460,9 +1490,7 @@ void NativeInlineTableVectorTest() {
   TestNativeInlineTableT unpacked;
   root->UnPackTo(&unpacked);
 
-  for (int i = 0; i < 10; ++i) {
-    TEST_ASSERT(unpacked.t[i] == test.t[i]);
-  }
+  for (int i = 0; i < 10; ++i) { TEST_ASSERT(unpacked.t[i] == test.t[i]); }
 
   TEST_ASSERT(unpacked.t == test.t);
 }
@@ -1557,6 +1585,7 @@ int FlatBufferTests(const std::string &tests_data_path) {
   ParseIncorrectMonsterJsonTest(tests_data_path);
   FixedLengthArraySpanTest(tests_data_path);
   DoNotRequireEofTest(tests_data_path);
+  JsonUnionStructTest();
 #else
   // Guard against -Wunused-parameter.
   (void)tests_data_path;
@@ -1621,6 +1650,7 @@ int FlatBufferTests(const std::string &tests_data_path) {
   StructKeyInStructTest();
   NestedStructKeyInStructTest();
   FixedSizedStructArrayKeyInStructTest();
+  EmbeddedSchemaAccess();
   return 0;
 }
 }  // namespace
